/* Name: main.c
 * Project: PowerSwitch based on AVR USB driver
 * Author: Christian Starkjohann
 * Creation Date: 2005-01-16
 * Tabsize: 4
 * Copyright: (c) 2005 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt) or proprietary (CommercialLicense.txt)
 * This Revision: $Id: main.c,v 1.4 2007/08/10 06:49:23 iruka Exp $
 */

/******************************************************************
 *				traRg[]TvvO.
 ******************************************************************
 *	
 *	
 *	
 *	
 */


/******************************************************************
 *	include
 ******************************************************************
 */

#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>

#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"

/******************************************************************
 *	Device / Config Descriptor
 ******************************************************************
 */
uchar usbFunctionDescriptor(usbRequest_t *rq);
#include "usbdesc.h"

typedef	struct {
	uchar bmRequestType;
	uchar cmd;
	uchar *adr;

	uchar data0;
	uchar data1;
	int   len;
} RxBuf;

//	cmd = [ 0 aa ccccc ]   a = area , c = cmd

enum {
	CMD_NOP  ,
	CMD_PEEK ,
	CMD_POKE ,
	CMD_READ ,
	CMD_MASK = 0x1f,
};

enum {
	AREA_RAM    = 0   ,
	AREA_EEPROM = 0x20,
	AREA_PGMEM  = 0x40,
//	reserved      0x60
	AREA_MASK	= 0x60,
};


static uchar txbuf[8];

#if	LED_DEBUG
/******************************************************************
 *	LED
 ******************************************************************
 */
void set_LED(int b01)
{
	PORTD = (PORTD & ~3) | b01;
}
#endif

void pgmemcpy(uchar *txbuf,int adr,uchar size);


//	attiny2313
static void eepromWrite(uchar addr, uchar val)
{
    while(EECR & (1 << EEPE));
    EECR = 0;
    EEAR = addr;
    EEDR = val;
    cli();
    EECR |= 1 << EEMPE;
    EECR |= 1 << EEPE;  /* must follow within a couple of cycles -- therefore cli() */
    sei();
}

static uchar    eepromRead(uchar addr)
{
    while(EECR & (1 << EEPE));
    EEAR = addr;
    EECR |= 1 << EERE;
    return EEDR;
}

void epmemcpy(uchar *txbuf,uchar addr,uchar size)
{
	do {
		*txbuf++ = eepromRead(addr);
		addr++;
		size--;
	}while(size);
}

/******************************************************************
 *	
 ******************************************************************
 */
static void cmdPeek(RxBuf *q)
{
	uchar sub = q->cmd & AREA_MASK;
	if(sub == AREA_RAM) {
		memcpy(txbuf,q->adr,q->data0);
	}else
	if(sub == AREA_EEPROM) {
		epmemcpy(txbuf,(uchar)q->adr,q->data0);
	}else
	if(sub == AREA_PGMEM) {
		pgmemcpy(txbuf,(int)q->adr,q->data0);
	}
}

/******************************************************************
 *	
 ******************************************************************
 */
static void cmdPoke(RxBuf *q)
{
	uchar *p = (uchar *)q->adr;
	if((q->cmd & AREA_MASK) == AREA_EEPROM) {
		eepromWrite((uchar)q->adr,q->data0);
		return;
	}

	if(q->data1) {	//}XNt̏.
		*p = (*p & q->data1) | q->data0;
	}else{			//ׂ.
		*p = q->data0;
	}
}


/******************************************************************
 *	ftHgȊÕRg[]ɌĂяoGg[.
 ******************************************************************
 *	x_NGXg̓NXNGXgɉ.
 *	   :usbRequset_t *rq;      NGXgpPbg\.
 *          ڍׂ usbdrv.h Q.
 *			rq->bRequest ɃR}hԍ邱Ƃ.
 *	߂l :uchar         replyLen 
 *			oCgԂꍇ usbMsgPtrɃAhXĂ.
 *			0xff Ԃꍇ usbFunctionRead()Ăяo.
 *
 */
uchar usbFunctionSetup(uchar data[8])
{
    usbMsgPtr = txbuf;

	RxBuf *q = (RxBuf *)data;
	uchar cmd = q->cmd & CMD_MASK;
	if(cmd == CMD_PEEK) {
		cmdPeek(q);
		return 8;
	}
	if(cmd == CMD_POKE) {
		cmdPoke(q);
		return 8;
	}
	return 0;
}


#if	0
/******************************************************************
 *	Control-InsɌĂ΂.
 ******************************************************************
 *	߂ĺAzXgɓ]oCg.(<= 8)
 *				0xff̎stall
 *	ꂪĂтɂ́A܂ usbFunctionSetup() ̕Ԃl 
 *  0xffłȂ΂ȂȂ.
 */
uchar usbFunctionRead( uchar *data, uchar len )
{
	
//			set_LED(0);
    return 0;
}
/******************************************************************
 *	Control-OutsɌĂ΂.
 ******************************************************************
 */
uchar usbFunctionWrite( uchar *data, uchar len )
{
	
//			set_LED(1);
    return 1;
}
/******************************************************************
 *	HOST->DEVICE:(Bulk-out) sɌĂ΂.
 ******************************************************************
 */
void usbFunctionWriteOut( uchar *data, uchar len )
{
	RxBuf *q = (RxBuf *)data;
	uchar cmd = q->cmd & CMD_MASK;
	if(cmd == CMD_PEEK) {
		cmdPeek(q);
		return;
	}
	if(cmd == CMD_POKE) {
		cmdPoke(q);
		return;
	}
}
#endif

/******************************************************************
 *
 ******************************************************************
 */
static void hardwareInit(void)
{
	// c{Ac|OɂB
	USB_CFG_IOPORT = ~((1<<USB_CFG_DMINUS_BIT)|(1<<USB_CFG_DPLUS_BIT));
	/* USB_CFG_IOPORT  PORTD Ɠ : ܂ PORTD = 0xf3 Ɠ */

	// PORTBׂē̓[hɂ.
	DDRB = 0;


// D݂ɍ킹Đ؂ւĂ.
#if	1
	// c{Ac|́A͏o̓[hɃZbgB
	USBDDR    = ~((1<<USB_CFG_DMINUS_BIT)|(1<<USB_CFG_DPLUS_BIT));
	/* USBDDR  DDRD Ɠ : ܂ DDRD = 0xf3 Ɠ */
#else
	// c{Ac|܂߁APORTDׂē̓[hɂB
	USBDDR    = 0;
#endif

}


/******************************************************************
 *
 ******************************************************************
 */
int main(void)
{
//	odDebugInit();
	hardwareInit();
	usbInit();
#if	0
	USB_SET_DATATOKEN1(USBPID_DATA1);
#endif
	sei();
	for(;;){
		usbPoll();
#if	0
		if( usbInterruptIsReady() ) {	// Send to Host.
			if( txque ) {
				usbSetInterrupt(txbuf, 8);
				txque--;
			}
		}
#endif
	}
}

